;
; Velleman K8048 PIC16F627 Debug Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: debug.asm,v 1.3 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC16F627
#INCLUDE        "p16f627.inc"
;*****************************************************************************
;
;   k8048 PIC to HOST communications
;
;   k8048 (PGD) -> host (CTS)
;   k8048 (PGC) <- host (RTS)
;
;   PIC and host perform a simple handshake operation, then the PIC outputs
;   a byte of data clocked by the host. If the host is not available then the
;   PIC will perform this operation unclocked.
;
;   Eg. on host:
;   k8048$ r
;   Hello, world
;   Hello, world
;   Hello, world
;   Hello, world
;   ^C
;
;*****************************************************************************
;
; Configuration word
;
    __CONFIG   _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _XT_OSC
;
; Numerical Constants (DEFINES)
;
STX             EQU     0x02
ETX             EQU     0x03
DELAYTIMH       EQU     0x20
WAITTIMH        EQU     0x00
;
; Data Constants (EEPROM)
;
                ORG     0x2100
                DE      "Hello, world\n",0
;
; Variables
;
DELAYTIMER1     EQU     0x20
DELAYTIMER2     EQU     0x21
WAITTIMER1      EQU     0x22
WAITTIMER2      EQU     0x23
OUTPUTBYTE      EQU     0x24
OUTPUTCOUNTER   EQU     0x25
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; test BIT in REG RETURN if set
RETBITSET       MACRO   REG,BIT
                BTFSC   REG,BIT
                RETURN
                ENDM
;
; test BIT in REG RETURN if clear
RETBITCLR       MACRO   REG,BIT
                BTFSS   REG,BIT
                RETURN
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
DELAYLOOP       MOVWF   DELAYTIMER1
DELAYLOOP1      CLRF    DELAYTIMER2
DELAYLOOP2      DJNZ    DELAYTIMER2,DELAYLOOP2
                DJNZ    DELAYTIMER1,DELAYLOOP1
                RETURN
;
; Wait for PGC Low
;
WAITLOW         MOVLW   WAITTIMH
                MOVWF   WAITTIMER1
WAITLOW1        CLRF    WAITTIMER2
WAITLOW2        RETBITCLR PORTB,6
                DJNZ    WAITTIMER2,WAITLOW2
                DJNZ    WAITTIMER1,WAITLOW1
                RETURN
;
; Wait for PGC High
;
WAITHIGH        MOVLW   WAITTIMH
                MOVWF   WAITTIMER1
WAITHGH1        CLRF    WAITTIMER2
WAITHGH2        RETBITSET PORTB,6
                DJNZ    WAITTIMER2,WAITHGH2
                DJNZ    WAITTIMER1,WAITHGH1
                RETURN
;
; Output byte
;
OUT             MOVWF   OUTPUTBYTE
                BSF     PORTB,0             ;SET LD1
;               
; handshake                
                CALL    WAITLOW             ;WAIT FOR LOW
                BSF     PORTB,7             ;SET PGD
                CALL    WAITHIGH            ;WAIT FOR HIGH
                BCF     PORTB,7             ;CLR PGD
;
; byte output
                MOVLW   0x08
                MOVWF   OUTPUTCOUNTER
                
OUT1            CALL    WAITLOW             ;WAIT FOR LOW

                CLRW
                RRF     OUTPUTBYTE,F        ;SHIFT OUT BIT
                SKPC
                MOVLW   B'10000000'

                MOVWF   PORTB

                CALL    WAITHIGH            ;WAIT FOR HIGH

                DJNZ    OUTPUTCOUNTER,OUT1  ;LOOP FOR 8 BITS
                
                BCF     PORTB,0             ;CLR LD1
                RETURN
;
; Output EEPROM Data String
;
OUTSTRING       BSF     STATUS,RP0          ;SELECT BANK 1
                MOVWF   EEADR               ;FIRST CHARACTER ADDRESS
                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   STX                 ;START OF TEXT RECORD
                CALL    OUT

OUTSTRING1      BSF     STATUS,RP0          ;SELECT BANK 1
                BSF     EECON1,RD           ;READ EEPROM DATA
                MOVF    EEDATA,W
                INCF    EEADR               ;NEXT CHARACTER ADDRESS
                BCF     STATUS,RP0          ;SELECT BANK 0

                IORLW   0                   ;TEST FOR END OF STRING
                BZ      OUTSTRING2

                CALL    OUT                 ;OUTPUT CURRENT CHARACTER
                GOTO    OUTSTRING1
                
OUTSTRING2      MOVLW   ETX                 ;END OF TEXT RECORD
                CALL    OUT
                RETURN
;
; Initialise PICMicro
;
INIT            BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                BSF     STATUS,RP0          ;SELECT BANK 1
                
                BCF     PCON,OSCF           ;INTRC OPERATION 37 KHz

                CLRF    INTCON              ;DISABLE INTERRUPTS

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE PULLUPS)
                MOVWF   OPTION_REG

                MOVLW   B'01000000'         ;PORT B PB7 OUT, PB6 IN
                MOVWF   TRISB

                MOVLW   B'11111111'         ;PORT A ALL I/P
                MOVWF   TRISA

                BCF     STATUS,RP0          ;SELECT BANK 0

                CLRF    PORTB               ;PB7 LOW

                RETURN
;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE
MAIN1           MOVLW   0x00                ;hello, world
                CALL    OUTSTRING
                CALL    DELAY

                GOTO    MAIN1               ;LOOP FOREVER
;
; Interrupt service routine (unused)
;
INTSR
                RETFIE
;
; Code End
;
                END
