;
; Velleman K8048 PIC16F627 VREF Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: vref.asm,v 1.3 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC16F627
#INCLUDE        "p16f627.inc"
;*****************************************************************************
;
; VREF Demo ramping voltage output from interrupt
;
;*****************************************************************************
;
; Configuration word
;
    __CONFIG    _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_CLKOUT
;
; Constants (DEFINES)
;
TIMERRELOAD     EQU     0x80

;
; Data Constants (EEPROM)
;

;
; Variables (RAM)
;
W_SAVE          EQU     0x70                ;IRQ SAVE
S_SAVE          EQU     0x71
VOLTAGE         EQU     0x72

;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR

;
; Start timer0
;
TIMERON         MOVLW   TIMERRELOAD
                MOVWF   TMR0

                MOVLW   0xA0
                MOVWF   INTCON              ;INT TMR0 ON
                RETURN

;
; Initialise PICMicro
;
INIT            BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                BSF     STATUS,RP0          ;SELECT BANK 1
                
                CLRF    INTCON              ;DISABLE INTERRUPTS

                BSF     PCON,OSCF           ;INTRC OPERATION 4 MHz

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE PULLUPS)
                MOVWF   OPTION_REG

                MOVLW   B'00000100'         ;PORT A 
                MOVWF   TRISA

                MOVLW   B'11101111'         ;PORT B
                MOVWF   TRISB

; VREF INIT
                MOVLW   B'11100000'         ;INIT VREF TO 0V
                MOVWF   VRCON
                CLRF    VOLTAGE
;
                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'11111111'         ;INIT PORT A
                MOVWF   PORTA

                CLRF    PORTB               ;INIT PORT B

                RETURN

;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE
                CALL    TIMERON
MAIN1           GOTO    MAIN1               ;LOOP FOREVER

;
; Interrupt service routine (unused)
;
INTSR           MOVWF   W_SAVE
                MOVF    STATUS,W
                MOVWF   S_SAVE

                BSF     STATUS,RP0          ;SELECT BANK 1
                BCF     STATUS,RP1

; ramp voltage
                INCF    VOLTAGE
                MOVF    VOLTAGE,W
                ANDLW   0x0F
                IORLW   0xE0
                MOVWF   VRCON

                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   TIMERRELOAD
                MOVWF   TMR0

                MOVF    S_SAVE,W
                MOVWF   STATUS
                MOVF    W_SAVE,W

                BCF     INTCON,2
                RETFIE

;
; Code End
;
                END
