;
; Velleman K8048 PIC16F627 Walsh Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: walsh.asm,v 1.3 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC16F627
#INCLUDE        "p16f627.inc"
;*****************************************************************************
;
; Demonstrate Walsh Functions on PB0(msb)->PB3(lsb) (R2R required) 
;
;*****************************************************************************
;
; Configuration word
;
    __CONFIG    _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_CLKOUT

;
; Constants (DEFINES)
;
WALSH1          EQU     B'11110000'         ;WALSH FUNCTION BIT STREAMS
WALSH2          EQU     B'11000011'
WALSH3          EQU     B'11001100'
WALSH6          EQU     B'10100101'
WALSH7          EQU     B'10101010'

TIMERRELOAD     EQU     0x80                ;SAMPLE RATE

;
; Variables (RAM)
;
OUT_1000        EQU     0x20                ;ROTATED OUTPUT STREAMS
OUT_0500        EQU     0x21
OUT_0250        EQU     0x22
OUT_0125        EQU     0x23

BUF_1000        EQU     0x24                ;OUTPUT STREAM RELOAD CACHE
BUF_0500        EQU     0x25
BUF_0250        EQU     0x26
BUF_0125        EQU     0x27

COUNTER         EQU     0x28                ;BIT COUNTER
PORTBITS        EQU     0x29                ;OUTPUT BITS

W_SAVE          EQU     0x70                ;IRQ SAVE
S_SAVE          EQU     0x71

;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR

;
; SQUARE
;
; WALSH1 x 1.000 *
; WALSH2 x 0.000
; WALSH3 x 0.000
; WALSH6 x 0.000
; WALSH7 x 0.000
;
SQUARE          MOVLW   WALSH1
                MOVWF   BUF_1000
                CLRF    BUF_0500
                CLRF    BUF_0250
                CLRF    BUF_0125
                RETURN

;
; SAWTOOTH
;
; WALSH1 x 0.500 *
; WALSH2 x 0.000
; WALSH3 x 0.250 *
; WALSH6 x 0.000
; WALSH7 x 0.125 *
;
SAWTOOTH        MOVLW   WALSH1
                MOVWF   BUF_0500
                MOVLW   WALSH3
                MOVWF   BUF_0250
                MOVLW   WALSH7
                MOVWF   BUF_0125
                CLRF    BUF_1000
                RETURN

;
; TRIANGLE
;
; WALSH1 x 0.250 *
; WALSH2 x 1.000 *
; WALSH3 x 0.000
; WALSH6 x 0.500 *
; WALSH7 x 0.000
;
TRIANGLE        MOVLW   WALSH1
                MOVWF   BUF_0250
                MOVLW   WALSH2
                MOVWF   BUF_1000
                MOVLW   WALSH6
                MOVWF   BUF_0500
                CLRF    BUF_0125
                RETURN

;
; Start timer0
;
TIMERON         CLRF    COUNTER

                MOVLW   TIMERRELOAD
                MOVWF   TMR0

                MOVLW   0xA0
                MOVWF   INTCON              ;INT TMR0 ON
                RETURN

;
; Initialise PICMicro
;
INIT            BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                BSF     STATUS,RP0          ;SELECT BANK 1
                
                CLRF    INTCON              ;DISABLE INTERRUPTS

                BSF     PCON,3              ;INIT INTRC OSC (0=37KHz, 1=4MHz)

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE PULLUPS)
                MOVWF   OPTION_REG

                MOVLW   B'00000000'         ;PORT B
                MOVWF   TRISB

                BCF     STATUS,RP0          ;SELECT BANK 0

                CLRF    PORTB               ;INIT PORT B
                RETURN

;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE
                CALL    TRIANGLE
                CALL    TIMERON
MAIN0           GOTO    MAIN0               ;LOOP FOREVER

;
; Interrupt service routine
;
INTSR           MOVWF   W_SAVE
                MOVF    STATUS,W
                MOVWF   S_SAVE

                BCF     STATUS,RP0
                BCF     STATUS,RP1
;
; output bits
;
                CLRF    PORTBITS
 
                RRF     OUT_0125,F
                RLF     PORTBITS,F

                RRF     OUT_0250,F
                RLF     PORTBITS,F

                RRF     OUT_0500,F
                RLF     PORTBITS,F

                RRF     OUT_1000,F
                RLF     PORTBITS,F

                MOVF    PORTBITS,W
                MOVWF   PORTB
;
; update counter
;
                INCF    COUNTER
                BTFSS   COUNTER,3
                GOTO    RESETTIMER
;    
; reload output streams
;
                MOVF    BUF_1000,W
                MOVWF   OUT_1000
                MOVF    BUF_0500,W
                MOVWF   OUT_0500
                MOVF    BUF_0250,W
                MOVWF   OUT_0250
                MOVF    BUF_0125,W
                MOVWF   OUT_0125
                CLRF    COUNTER
;
; reset timer
;
RESETTIMER      MOVLW   TIMERRELOAD
                MOVWF   TMR0

                MOVF    S_SAVE,W
                MOVWF   STATUS
                MOVF    W_SAVE,W

                BCF     INTCON,2
                RETFIE

;
; Code End
;
                END
