;
; Velleman K8048 PIC16F648A Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: demo.asm,v 1.4 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC16F648A
#INCLUDE        "p16f648a.inc"
;*****************************************************************************
;
;   k8048 PIC16F648A Demo
;
;*****************************************************************************
;
; Configuration word
;
              __CONFIG  _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;             __CONFIG  _CP_ON  & _DATA_CP_ON  & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;
; Numerical Constants (DEFINES)
;
DELAYTIMH       EQU     0x80
;
; Data Constants (USERID)
;
                ORG     0x2000
                DE      "PIC!"
;
; Data Constants (EEPROM)
;
                ORG     0x2100
                DE      "EEPROM WRITE TEST"
;
; Variables
;
DELAYTIMER1     EQU     0x20
DELAYTIMER2     EQU     0x21
OUTPUTBYTE      EQU     0x22
OUTPUTEOR       EQU     0x23
OUTPUTCNT       EQU     0x24
;
W_SAVE          EQU     0x70
S_SAVE          EQU     0x71
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; test BIT in REG, goto LOC if CLR
BBFC            MACRO   REG,BIT,LOC
                BTFSS   REG,BIT
                GOTO    LOC
                ENDM
;
; ISR Save
ISRSAVE         MACRO   
                MOVWF   W_SAVE
                MOVF    STATUS,W
                MOVWF   S_SAVE
                BCF     STATUS,RP0
                BCF     STATUS,RP1
                ENDM
;
; ISR Restore
ISRRESTORE      MACRO
                MOVF    S_SAVE,W
                MOVWF   STATUS
                MOVF    W_SAVE,W
                BCF     INTCON,2
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
DELAYLOOP       MOVWF   DELAYTIMER1
DELAYLOOP1      CLRF    DELAYTIMER2
DELAYLOOP2      DJNZ    DELAYTIMER2,DELAYLOOP2
                DJNZ    DELAYTIMER1,DELAYLOOP1
                RETURN
;
; Initialise PICMicro
;
INIT            BSF     STATUS,RP0          ;SELECT BANK 1

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE PULLUPS)
                MOVWF   OPTION_REG

                MOVLW   B'11111111'         ;PORT A SW1..SW4 I/P
                MOVWF   TRISA
                
                MOVLW   B'11000000'         ;PORT B LD1..LD6 O/P
                MOVWF   TRISB

                CLRF    INTCON              ;DISABLE INTERRUPTS

                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON
;
; Init LEDs
;
                CLRF    PORTB
                CLRF    OUTPUTBYTE
                CLRF    OUTPUTEOR
;
; Init Timer 1
;
                CLRF    PIR1                ;CLEAR PERIPHERAL IRQ FLAGS
                CLRF    T1CON               ;RESET COUNTER CONTROL

                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   0x01
                MOVWF   PIE1                ;ENABLE OVERFLOW IRQ

                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   0xc0
                MOVWF   INTCON              ;ENABLE GIE + PEIE

                MOVLW   0x01
                MOVWF   T1CON               ;START TIMER
                RETURN
;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE

MAIN1           BTFSC   PORTA,0             ;DETECT KEY PRESS
                GOTO    SW1
                BTFSC   PORTA,1
                GOTO    SW2
                BTFSC   PORTA,2
                GOTO    SW3
                BTFSC   PORTA,3
                GOTO    SW4
                GOTO    MAIN1

SW1             MOVLW   0x00                ;SW1 PRESSED
                MOVWF   OUTPUTEOR
                GOTO    MAIN1
SW2             MOVLW   0xFF                ;SW2 PRESSED
                MOVWF   OUTPUTEOR
                GOTO    MAIN1
SW3             MOVLW   0xF8                ;SW3 PRESSED
                MOVWF   OUTPUTEOR
                GOTO    MAIN1
SW4             MOVLW   0x07                ;SW4 PRESSED
                MOVWF   OUTPUTEOR
                GOTO    MAIN1
;
; Interrupt service routine
;
INTSR           ISRSAVE
                CLRF    PIR1                ;CLEAR PERIPHERAL IRQ FLAGS

                INCF    OUTPUTCNT           ;SCALER
                BTFSS   OUTPUTCNT,3
                GOTO    RESTORE
                CLRF    OUTPUTCNT

                INCF    OUTPUTBYTE          ;UPDATE
                BTFSS   OUTPUTBYTE,6
                GOTO    UPDATE
                CLRF    OUTPUTBYTE
UPDATE          MOVF    OUTPUTBYTE,W
                XORWF   OUTPUTEOR,W
                MOVWF   PORTB
                
RESTORE         ISRRESTORE
                RETFIE
;
; Code End
;
                END
