;
; Velleman K8048 PIC16F648A RS-232 Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: rs232.asm,v 1.11 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC16F648A
#INCLUDE        "p16f648a.inc"
;*****************************************************************************
;
;   k8048 PIC16F648A RS-232 Demo
;
;   PIC TX-OUT connected to 14P DIL socket PIN8 (RB2)
;   PIC RX-IN  connected to 14P DIL socket PIN9 (RB1)
;
;   External lines utilise ST232CN (MAX232) for level conversion
;
;   Demo:
;    1. TODO Receive line terminated by <CR><LF> (up to 8 bytes in length including <CR><LF>)
;    2. Do nothing in particular
;    3. Acknowledge received line with ACK<CR><LF>
;
;   Test with UNIX callup (cu):
;
;    cu -l /dev/ttyUSB0 -s 9600
;  
;*****************************************************************************
;
; Configuration word
;
              __CONFIG  _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;             __CONFIG  _CP_ON  & _DATA_CP_ON  & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;
; Numerical Constants (DEFINES)
;
DELAYTIML       EQU     0x00
DELAYTIMM       EQU     0x00
DELAYTIMH       EQU     0x02
;
; Data Constants (USERID)
;
                ORG     0x2000
                DE      "UART"
;
; Data Constants (EEPROM)
;
                ORG     0x2100
                DE      "UART"
;
; Variables
;
; GLOBAL
W_SAVE          EQU     0x70    ;INTERRUPT SAVES
S_SAVE          EQU     0x71
F_SAVE          EQU     0x72
;
COUNTER1        EQU     0x78    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x79
COUNTER3        EQU     0x7a
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            BCF     STATUS,RP0          ;SELECT BANK 0
                BCF     STATUS,RP1
                BCF     STATUS,IRP
                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON
                CLRF    INTCON              ;DISABLE INTERRUPTS

                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE INPUT PULLUPS + TMR0 DEFAULTS)
                MOVWF   OPTION_REG
                CLRF    PIE1                ;DISABLE PERIPHERAL INTERRUPTS
                MOVLW   B'11111111'
                MOVWF   TRISA               ;PORT A INP
                MOVLW   B'11111111'
                MOVWF   TRISB               ;PORT B INP
;
;setup UART 9600 baud
;
                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   .25
                MOVWF   SPBRG               ;ASYNC,BRGH=1,9600(9615)(FOSC=4MHz)

                BCF     STATUS,RP0          ;SELECT BANK 0
                MOVLW   (1 << SPEN) | (1 << CREN)
                MOVWF   RCSTA               ;SERIAL + RECEIVER ENABLE

                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   (1 << TXEN) | (1 << BRGH)
                MOVWF   TXSTA               ;TRANSMITTER ENABLE + HIGH SPEED
                RETURN
;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE

MAIN1           BCF     STATUS,RP0          ;SELECT BANK 0
                
                CALL    RXWAIT

                CALL    TXWAIT              ;ECHO
                MOVWF   TXREG

                CALL    TXWAIT
                MOVLW   'A'
                MOVWF   TXREG               ;TRANSMIT ACK<CR><LF>

                CALL    TXWAIT
                MOVLW   'C'
                MOVWF   TXREG

                CALL    TXWAIT
                MOVLW   'K'
                MOVWF   TXREG

                CALL    TXWAIT
                MOVLW   0x0d
                MOVWF   TXREG
                
                CALL    TXWAIT
                MOVLW   0x0a
                MOVWF   TXREG

                GOTO    MAIN1

TXWAIT          BTFSS   PIR1,TXIF
                GOTO    TXWAIT
                RETURN

RXWAIT          BTFSS   PIR1,RCIF
                GOTO    RXWAIT

                BTFSC   RCSTA,OERR
                GOTO    RXERR
                BTFSC   RCSTA,FERR
                GOTO    RXERR

                MOVF    RCREG,W
                RETURN

RXERR           BCF     RCSTA,CREN
                MOVLW   (1 << SPEN) | (1 << CREN)
                MOVWF   RCSTA
                GOTO    RXWAIT
;               
;
; Interrupt service routine
;
INTSR           MOVWF   W_SAVE              ;SAVE WORKING REGISTER
                MOVF    STATUS,W
                MOVWF   S_SAVE              ;SAVE STATUS
                MOVF    FSR,W
                MOVWF   F_SAVE              ;SAVE FSR

INTSRRESTORE    MOVF    F_SAVE,W            ;RESTORE FSR
                MOVWF   FSR
                MOVF    S_SAVE,W            ;RESTORE STATUS
                MOVWF   STATUS
                MOVF    W_SAVE,W            ;RESTORE WORKING REGISTER
                RETFIE
;
;
; Code End
;
                END
