;
; Velleman K8048 PIC16F648A Sky RC6 Remote Control Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: skyrc6.asm,v 1.21 2007/06/02 02:41:47 darron Exp $
;
                LIST    P=PIC16F648A
#INCLUDE        "p16f648a.inc"
;*****************************************************************************
;
;   k8048 PIC16F648A Sky RC6 Remote Control Demo
;
;   See:
;
;   RC6 overview        http://www.sbprojects.com/knowledge/ir/rc6.htm
;   Sky RC6 (RC-6-20)   http://john.fine.home.comcast.net/ir/DecodeIr.html
;   Sky magic eye       http://groups.google.co.uk/group/uk.tech.tv.sky/browse_thread/thread/616289ac2ef88dc4/1ad625df42dc9de0
;   Sky key codes       http://www.dusky-control.com/sky-control-codes.shtml
;
;   Demo: SKY BLUE BUTTON (00-0-70)
;
;                   4  2  1         8  4  2  1   8  4  2  1   8  4  2  1   8  4  2  1   8  4  2  1
;    -- -- -- __ -_ -_ -_ _- __ --  _- _- _- _-  _- _- _- _-  _- _- _- _-  _- -_ -_ -_  _- _- _- _-
;    YY YY YY XX YX YX YX XY XX YY  XY XY XY XY  XY XY XY XY  YX YX XY XY  XY YX YX YX  XY XY XY XY
;                                    0  0  0  0   0  0  0  0   0  0  0  0   0  1  1  1   0  0  0  0  00-0-70
;    ===LEADER== SB ==MODE== =TR==  ==CONTROL ADDRESS=======  ==INFORMATION COMMAND================
;
;   X=OFF Y=ON XY=0 YX=1 SB=START-BIT TR=TRAILER-BIT
;
;   BITSTREAM (MANCHESTER ENCODED BYTES)
;  
;    PRE     [0] F C A 9 3              [0]=SPACER [FC]=LEADER [A9]=SB+MODE [3]=TRAILER = 0f ca 93
;    ADDRESS     5 5 5 5
;    DATA        5 5 6 A 5 5 (BLUE)
;
;   TEST EMITTER SETUP ON K8048 BOARD
;
;   1 x VISHAY TSAL4400 connected to 14P DIL socket PIN7 (RB3) tied to ground via 390R
;
;   XT_OSC = 4MHz
;
;   RB3 MAY ALSO HAVE A PULL DOWN RESISTOR (EG. 1M) IF DRIVING LOGIC
;  
;*****************************************************************************
;
; Configuration word
;
              __CONFIG  _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;             __CONFIG  _CP_ON  & _DATA_CP_ON  & _LVP_OFF & _BODEN_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _XT_OSC
;
; Numerical Constants (DEFINES)
;
DELAYTIML       EQU     0x00
DELAYTIMM       EQU     0x00
DELAYTIMH       EQU     0x20
;
; PWM MARK TIME (LOW BITS) + PWM ENABLE (SPACE TIME/2)
PWMMLEN         EQU     0x3c
; PWM MARK TIME (HIGH BITS)
PWMMH           EQU     0x0d
; PWM SPACE TIME (HIGH BITS) (1MHz / 36000)
PWMSH           EQU     0x1b 
;
; RB3 CCP1 OUTPUT ENABLE : RB3=PWM         (TRISB) (RB4 IS MODULATOR TEST POINT)
;                         76543210
PBCCP1EN        EQU     B'11100111'
;
; RB3 CCP1 OUTPUT DISABLE: RB3=TRI-STATE   (TRISB) (PULLUPS DISABLED)
;                         76543210
PBCCP1DIS       EQU     B'11111111'
;
; TMR0 RELOAD 256 - (444uS - RELOAD DELAY) / 2
TMR0RELOAD      EQU     0x29
;
; Data Constants (USERID)
;
                ORG     0x2000
                DE      "Sky!"
;
; Data Constants (EEPROM)
;
                ORG     0x2100
                DE      "Sky!"
;
; Variables
;
; BANK1
RCBUFPRE0       EQU     0xa0    ;PRE  [0FCA93]  sky RC6 constant header
RCBUFPRE1       EQU     0xa1
RCBUFPRE2       EQU     0xa2
RCBUFADDR0      EQU     0xa3    ;ADDR [5555]    sky RC6 constant address
RCBUFADDR1      EQU     0xa4
RCBUFDATA0      EQU     0xa5    ;DATA [55]      sky RC6 device code 55=0=SKY A5=12=SKY+
RCBUFDATA1      EQU     0xa6    ;DATA [XX]      sky key code high nibble
RCBUFDATA2      EQU     0xa7    ;DATA [XX]      sky key code low  nibble
RCBUFBYTIND     EQU     0xa8    ;BUFFER INDIRECT INDEX
RCBUFBITCNT     EQU     0xa9    ;BIT COUNTER FOR CURRENTLY INDEXED BYTE
;
; GLOBAL
W_SAVE          EQU     0x70    ;INTERRUPT SAVES
S_SAVE          EQU     0x71
F_SAVE          EQU     0x72
;
COUNTER1        EQU     0x78    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x79
COUNTER3        EQU     0x7a
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            BCF     STATUS,RP0          ;SELECT BANK 0
                BCF     STATUS,RP1
                BCF     STATUS,IRP

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                CLRF    INTCON              ;DISABLE INTERRUPTS
                CLRF    PIR1                ;CLEAR INTERRRUPT FLAGS

                MOVLW   0x10                ;INIT TEST POINT RB4
                MOVWF   PORTB

                BSF     STATUS,RP0          ;SELECT BANK 1

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE INPUT PULLUPS + TMR0 DEFAULTS)
                MOVWF   OPTION_REG

                CLRF    PIE1                ;DISABLE PERIPHERAL INTERRUPTS

                MOVLW   B'11111111'
                MOVWF   TRISA               ;PORT A INP
;
;setup output buffer (empty)
;
                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   0x08                ;BUFFER EMPTY
                MOVWF   RCBUFBYTIND 
                MOVLW   0x08
                MOVWF   RCBUFBITCNT
;
;setup PWM @ approx 36 KHz (CCPR1L=HIGH TIME, PR2=LOW TIME, 1MHz/36000 = 27u = 0x1b)
;
                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   PBCCP1DIS
                MOVWF   TRISB

                BCF     STATUS,RP0          ;SELECT BANK 0
                MOVLW   PWMMLEN             ;PWM MARK (LOW BITS) + ENABLE
                MOVWF   CCP1CON
                MOVLW   PWMMH               ;PWM MARK (HIGH BITS)
                MOVWF   CCPR1L
                
                BSF     STATUS,RP0          ;SELECT BANK 1
                MOVLW   PWMSH               ;PWM SPACE (HIGH BITS)
                MOVWF   PR2

                BCF     STATUS,RP0          ;SELECT BANK 0
                MOVLW   0x04                ;TMR2 ON
                MOVWF   T2CON
;
;setup timer0 interrupt (444us)
;
                MOVLW   TMR0RELOAD          ;INIT TMR0
                MOVWF   TMR0
                MOVLW   0xa0                ;ENABLE INTERRUPTS TMR0 
                MOVWF   INTCON

                RETURN
;
; Main program (send BLUE button repeatedly)
;
MAIN            CALL    INIT                ;INITIALISE
                
                BSF     STATUS,RP0          ;SELECT BANK 1

MAIN1           MOVLW   0x0f                ;PRE [0fca93]
                MOVWF   RCBUFPRE0
                MOVLW   0xca
                MOVWF   RCBUFPRE1
                MOVLW   0x93
                MOVWF   RCBUFPRE2

                MOVLW   0x55                ;ADDR [5555] = [00]
                MOVWF   RCBUFADDR0
                MOVLW   0x55
                MOVWF   RCBUFADDR1

                MOVLW   0x55                ;DATA [556a55] = [070]
                MOVWF   RCBUFDATA0
                MOVLW   0x6a
                MOVWF   RCBUFDATA1
                MOVLW   0x55
                MOVWF   RCBUFDATA2

                MOVLW   RCBUFPRE0           ;SEND
                MOVWF   RCBUFBYTIND

MAIN2           BTFSS   RCBUFBYTIND,3       ;WAIT FOR SEND COMPLETION
                GOTO    MAIN2

MAIN3           CALL    DELAY
                GOTO    MAIN1
;
; Interrupt service routine entered every 444uS (approx)
;
INTSR           MOVWF   W_SAVE              ;SAVE WORKING REGISTER
                MOVF    STATUS,W
                MOVWF   S_SAVE              ;SAVE STATUS
                MOVF    FSR,W
                MOVWF   F_SAVE              ;SAVE FSR

                BCF     STATUS,RP0          ;SELECT BANK 0
                MOVLW   TMR0RELOAD          ;RELOAD TMR0
                MOVWF   TMR0                ;RELOAD DELAY = 8(HERE) + 2(TMR STALL) + 4(TCY)

                BSF     STATUS,RP0          ;SELECT BANK 1
                BTFSS   RCBUFBYTIND,3       ;BUFFER EMPTY?
                GOTO    SENDBIT             ;NO,  SEND ANOTHER BIT...
                MOVLW   PBCCP1DIS           ;YES, DISABLE (TRI-STATE) OUTPUT
                MOVWF   TRISB
                GOTO    INTSRRESTORE

SENDBIT         MOVF    RCBUFBYTIND,W       ;GET BYTE INDEX
                MOVWF   FSR                 
                MOVLW   PBCCP1DIS           ;DISABLE (TRI-STATE) OUTPUT
                RLF     INDF,F              ;GET BIT
                BTFSC   STATUS,C
                MOVLW   PBCCP1EN            ;ENABLE (PWM) OUTPUT
                MOVWF   TRISB               ;PASS THROUGH PWM 36KHz TONE ELSE TRI-STATE

                DECFSZ  RCBUFBITCNT,F       ;NEXT BIT
                GOTO    INTSRRESTORE
                MOVLW   0x08
                MOVWF   RCBUFBITCNT
                INCF    RCBUFBYTIND,F       ;NEXT BYTE

INTSRRESTORE    MOVF    F_SAVE,W            ;RESTORE FSR
                MOVWF   FSR
                MOVF    S_SAVE,W            ;RESTORE STATUS
                MOVWF   STATUS
                MOVF    W_SAVE,W            ;RESTORE WORKING REGISTER
                BCF     INTCON,2            ;RESET TMR0 INT FLAG
                RETFIE
;
; Code End
;
                END
