;
; Velleman K8048 PWM Demo (Close encounters!)
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: pwm.asm,v 1.9 2007/06/06 23:28:01 darron Exp $
;
                LIST    P=PIC18F1320
#INCLUDE        "p18f1320.inc"
;*****************************************************************************
;
;   k8048 PIC18F1320 PWM Demo (Close encounters!)
;
;   This demonstrates with the aide of two external devices how to get the
;   pic micro to provide an almost full range of midi notes utilising the
;   pwm. See for pwm.sch for connexions.
;
;   The accompanying .c file shows how to generate the necessary
;   counter and post scaling details for a given oscillator frequency and
;   timer prescaler.
;
;   XTAL=10MHz/CLK=40Mhz/CPU CLK=10MHz
;
;*****************************************************************************
;
; Configuration words
;
              __CONFIG _CONFIG1H, _IESO_OFF_1H  & _FSCM_OFF_1H & _HSPLL_OSC_1H

              __CONFIG _CONFIG2L, _BORV_20_2L   & _BOR_OFF_2L  & _PWRT_ON_2L
              __CONFIG _CONFIG2H, _WDTPS_32K_2H & _WDT_OFF_2H

              __CONFIG _CONFIG3H, _MCLRE_ON_3H

              __CONFIG _CONFIG4L, _DEBUG_OFF_4L & _LVP_OFF_4L  & _STVR_ON_4L

              __CONFIG _CONFIG5L, _CP1_OFF_5L   & _CP0_OFF_5L
              __CONFIG _CONFIG5H, _CPD_OFF_5H   & _CPB_OFF_5H
              
              __CONFIG _CONFIG6L, _WRT1_OFF_6L  & _WRT0_OFF_6L
              __CONFIG _CONFIG6H, _WRTD_OFF_6H  & _WRTB_OFF_6H & _WRTC_OFF_6H
              
              __CONFIG _CONFIG7L, _EBTR1_OFF_7L & _EBTR0_OFF_7L
              __CONFIG _CONFIG7H, _EBTRB_OFF_7H
;
; Numerical Constants (DEFINES)
;
DELAYTIML       EQU     0x00
DELAYTIMM       EQU     0x00
DELAYTIMH       EQU     0x28
;
; Data Constants (USERID)
;
                ORG     0x200000
                DB      0,1,2,3,4,5,6,7
;
; Data Constants (EEPROM)
;
                ORG     0xF00000
                DE      "PWM!"
;
; Variables
;
TEMP            EQU     0x28
;
COUNTER1        EQU     0x30    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x31
COUNTER3        EQU     0x32
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0008
                GOTO    INTHIGH
                ORG     0x0018
                GOTO    INTLOW
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            CLRF    ADCON0              ;A/D OFF
                MOVLW   0x7f
                MOVWF   ADCON1
                CLRF    ADCON2
;                
                CLRF    INTCON
                CLRF    PIR1
                CLRF    PORTB
                CLRF    TBLPTRU
;
;15.5.9 setup pwm
;
                CLRF    PORTB
                MOVLW   b'00000000'         ;PWM P1A OUTPUT ENABLED + MUX SELECT
                MOVWF   TRISB
                MOVLW   b'00001100'         ;PWM MODE ENABLED
                MOVWF   CCP1CON
                RETURN
;
; Play Midi Note
;
#INCLUDE        "notetable.inc"
NOTEON          MOVWF   TEMP                ;SAVE MIDI NOTE

                MOVLW   LOW(NOTETABLE)      ;TABLE BASE
                MOVWF   TBLPTRL
                MOVLW   HIGH(NOTETABLE)
                MOVWF   TBLPTRH
                MOVLW   NOTESTART           ;TABLE BASE= TABLE BASE + 2 * (NOTE - NOTESTART)
                SUBWF   TEMP,F
                MOVF    TEMP,W
                ADDWF   TBLPTRL,F
                CLRW
                ADDWFC  TBLPTRH,F
                MOVF    TEMP,W
                ADDWF   TBLPTRL,F
                CLRW
                ADDWFC  TBLPTRH,F
                
                CLRF    T2CON               ;OFF
                TBLRD*+
                MOVFF   TABLAT,PORTB        ;MUX SELECT
                TBLRD*
                MOVFF   TABLAT,PR2          ;MARK/SPACE TIME
                MOVFF   TABLAT,CCPR1L
                CLRC
                RRCF    CCPR1L,F            ;MARK TIME=MARK/SPACE TIME/2
                MOVLW   b'00000110'         ;TMR2 ON PRE/16
                MOVWF   T2CON
                RETURN
;
; Main program
;
MAIN            CALL    INIT

MAIN1
;               CLOSE ENCOUNTERS OF THE THIRD KIND
;
                MOVLW   0x3c        ;midi 60 (C)
                CALL    NOTEON
                CALL    DELAY
                MOVLW   0x3e        ;midi 62 (D)
                CALL    NOTEON
                CALL    DELAY
                MOVLW   0x3a        ;midi 58 (A#)
                CALL    NOTEON
                CALL    DELAY
                MOVLW   0x2e        ;midi 46 (A#)
                CALL    NOTEON
                CALL    DELAY
                MOVLW   0x35        ;midi 53 (F)
                CALL    NOTEON
                CALL    DELAY

                CLRF    T2CON       ;OFF
                CALL    DELAY
                GOTO    MAIN1
;
; Interrupt service routine (unused)
;
INTHIGH
INTLOW          RETFIE
;
; Code End
;
                END
