;
; Velleman K8048 Serial Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: serial.asm,v 1.4 2007/06/01 01:51:35 darron Exp $
;
                LIST    P=PIC18F1320
#INCLUDE        "p18f1320.inc"
;*****************************************************************************
;
;   k8048 PIC18F1320 Serial Demo
;
;   This demonstrates the PIC18F1320 communicating with two different
;   serial devices:
;
;   1. TLC5620 4 OUTPUT DAC
;   2. DS1267  PROGRAMMABLE POTENTIOMETER
;
;   BOTH THESE DEVICES USE NON STANDARD SERIAL INTERFACES AND IN THIS
;   DEMO THE DATA AND CLOCK LINES ARE SHARED BETWEEN THE TWO AND
;   THE CONTROL LINES ARE SEPERATE.
;
;   XTAL=10MHz/CLK=40Mhz
;
;   XXX DUE TO BCF/BSF BEING R/W/M THESE ROUTINES NEED VERIFICATION
;       ON TARGET TO DETERMINE IF I/O ROUTINES ARE SUCCESSFUL.
;       ON DEMO BREADBOARD THERE ARE NO ISSUES OTHER THAN A DELAY
;       AFTER SETTING RA4 OPEN DRAIN PULLUP PRIOR TO NEXT R/W/M
;       CYCLE.
;
;*****************************************************************************
;
; Configuration words
;
              __CONFIG _CONFIG1H, _IESO_OFF_1H  & _FSCM_OFF_1H & _HSPLL_OSC_1H

              __CONFIG _CONFIG2L, _BORV_20_2L   & _BOR_OFF_2L  & _PWRT_ON_2L
              __CONFIG _CONFIG2H, _WDTPS_32K_2H & _WDT_OFF_2H

              __CONFIG _CONFIG3H, _MCLRE_ON_3H

              __CONFIG _CONFIG4L, _DEBUG_OFF_4L & _LVP_OFF_4L  & _STVR_ON_4L

              __CONFIG _CONFIG5L, _CP1_OFF_5L   & _CP0_OFF_5L
              __CONFIG _CONFIG5H, _CPD_OFF_5H   & _CPB_OFF_5H
              
              __CONFIG _CONFIG6L, _WRT1_OFF_6L  & _WRT0_OFF_6L
              __CONFIG _CONFIG6H, _WRTD_OFF_6H  & _WRTB_OFF_6H & _WRTC_OFF_6H
              
              __CONFIG _CONFIG7L, _EBTR1_OFF_7L & _EBTR0_OFF_7L
              __CONFIG _CONFIG7H, _EBTRB_OFF_7H
;
; Numerical Constants (DEFINES)
;
DACADDRA        EQU     0x00                ;DAC A..D ADDRESS
DACADDRB        EQU     0x01
DACADDRC        EQU     0x02
DACADDRD        EQU     0x03
;
DELAYTIML       EQU     0x00
DELAYTIMM       EQU     0x00
DELAYTIMH       EQU     0x02
;
; Data Constants (USERID)
;
                ORG     0x200000
                DB      0,1,2,3,4,5,6,7
;
; Data Constants (EEPROM)
;
                ORG     0xF00000
                DE      "Serial!"
;
; Variables
;
TEMP1           EQU     0x20    ;VOLATILE
TEMP2           EQU     0x21    ;VOLATILE
;
DACA            EQU     0x22    ;DAC A SETTING
DACB            EQU     0x23    ;DAC B SETTING
DACC            EQU     0x24    ;DAC C SETTING
DACD            EQU     0x25    ;DAC D SETTING
;
POT0            EQU     0x26    ;POT0 SETTING
POT1            EQU     0x27    ;POT1 SETTING
;
COUNTER1        EQU     0x30    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x31
COUNTER3        EQU     0x32
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            CLRF    ADCON0              ;A/D OFF
                MOVLW   0x7f
                MOVWF   ADCON1
                CLRF    ADCON2

                MOVLW   b'00001100'         ;RESET !RST, SET LDAC, SET LOAD, RESET DATA, RESET CLK
                MOVWF   PORTA
                MOVLW   b'00000000'         ;RA4:!RST RA3:LDAC RA2:LOAD RA1:DATA RA0:CLK
                MOVWF   TRISA
                RETURN
;
; Set DAC
;
SETDAC          MOVWF   TEMP2
                BTFSC   TEMP2,1             ;A1
                BSF     PORTA,1
                BSF     PORTA,0             ;CLOCK IN A1
                BCF     PORTA,0
                BCF     PORTA,1             ;RESET DATA

                BTFSC   TEMP2,0             ;A0
                BSF     PORTA,1
                BSF     PORTA,0             ;CLOCK IN A0
                BCF     PORTA,0

                BSF     PORTA,1             ;RNG X2
                BSF     PORTA,0             ;CLOCK IN RNG
                BCF     PORTA,0
                BCF     PORTA,1             ;RESET DATA

                MOVLW   0x08                ;D7..D0
                MOVWF   TEMP2
DATA0           RLCF    TEMP1,F
                BNC     DATA1
                BSF     PORTA,1
DATA1           BSF     PORTA,0             ;CLOCK IN DN
                BCF     PORTA,0
                BCF     PORTA,1             ;RESET DATA
                DJNZ    TEMP2,DATA0

LOAD            BCF     PORTA,2
                NOP
                NOP
                BSF     PORTA,2
                RETURN
;
; Set DACs
;
SETDACS         MOVF    DACA,W
                MOVWF   TEMP1
                MOVLW   DACADDRA
                CALL    SETDAC

                MOVF    DACB,W
                MOVWF   TEMP1
                MOVLW   DACADDRB
                CALL    SETDAC
                
                MOVF    DACC,W
                MOVWF   TEMP1
                MOVLW   DACADDRC
                CALL    SETDAC
                
                MOVF    DACD,W
                MOVWF   TEMP1
                MOVLW   DACADDRD
                CALL    SETDAC

LDAC            BCF     PORTA,3
                NOP
                NOP
                BSF     PORTA,3
                RETURN
;
; Set POTs
;
SETPOTS         BSF     PORTA,4             ;RST HIGH (RA4 IS OPEN DRAIN, USE 2K2 PULLUP)
                NOP                         ;WAIT FOR PULLUP (READ/WRITE/MODIFY)
                NOP                         ; "    "    "
                NOP                         ; "    "    "

                BCF     PORTA,1             ;STACK SELECT DISABLE
                BSF     PORTA,0             ;CLOCK IN STACK SELECT
                BCF     PORTA,0
                
                MOVF    POT1,W
                MOVWF   TEMP1
                MOVLW   0x08
                MOVWF   TEMP2

SETPOT0         RLCF    TEMP1,F
                BNC     SETPOT1
                BSF     PORTA,1
SETPOT1         BSF     PORTA,0             ;CLOCK IN DATA
                BCF     PORTA,0
                BCF     PORTA,1             ;RESET DATA
                DJNZ    TEMP2,SETPOT0

                MOVF    POT0,W
                MOVWF   TEMP1
                MOVLW   0x08
                MOVWF   TEMP2

SETPOT2         RLCF    TEMP1,F
                BNC     SETPOT3
                BSF     PORTA,1
SETPOT3         BSF     PORTA,0             ;CLOCK IN DATA
                BCF     PORTA,0
                BCF     PORTA,1             ;RESET DATA
                DJNZ    TEMP2,SETPOT2

                BCF     PORTA,4             ;RST LOW
                RETURN
;
; Main program
;
MAIN            CALL    INIT
                
                CLRF    DACA
                CLRF    DACB
                CLRF    DACC
                CLRF    DACD

                CLRF    POT0
                CLRF    POT1

MAIN1           CALL    SETDACS
                CALL    SETPOTS

                INCF    DACA
                INCF    POT0
    
                CALL    DELAY

MAIN2           GOTO    MAIN1
;
; Interrupt service routine (unused)
;
INTSR           RETFIE
;
; Code End
;
                END
