;
; Velleman K8048 Triangle Wave Demo
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: triangle.asm,v 1.2 2007/05/29 15:26:46 darron Exp $
;
                LIST    P=PIC18F2550
#INCLUDE        "p18f2550.inc"
;*****************************************************************************
;
;   k8048 PIC18F2550 Triangle Demo
;
;   16 Mhz XTAL (48 MHz CPU/USB Clock)
;       FOSC            : HSPLL (16 MHz)
;       PLL PRESCALE    : /4    (96 MHz)
;       PLL POSTSCALE   : /2    (48 MhZ)
;
;   CIRCUIT
;
;   PIC
;    PB7\
;    .   \
;    .    >===DAC 0800===>SIGNAL
;    .   /
;    PB0/
;
;*****************************************************************************
;
; Configuration words
;
              __CONFIG _CONFIG1L, _USBDIV_2_1L    & _CPUDIV_OSC1_PLL2_1L & _PLLDIV_4_1L
              __CONFIG _CONFIG1H, _IESO_OFF_1H    & _FCMEM_OFF_1H        & _FOSC_HSPLL_HS_1H 

; The gpasm header changed the BOR voltage definition at some point in time...
                ifdef _BORV_21_2L
              __CONFIG _CONFIG2L, _VREGEN_OFF_2L  & _BORV_21_2L          & _BOR_OFF_2L     & _PWRT_ON_2L
                else
              __CONFIG _CONFIG2L, _VREGEN_OFF_2L  & _BORV_3_2L           & _BOR_OFF_2L     & _PWRT_ON_2L
                endif

              __CONFIG _CONFIG2H, _WDTPS_32768_2H & _WDT_OFF_2H

              __CONFIG _CONFIG3H, _MCLRE_ON_3H    & _LPT1OSC_OFF_3H      & _PBADEN_OFF_3H  & _CCP2MX_OFF_3H

; Microchip datasheet DS39632D page 286 explains that ICPTR should be CLEAR when N/A rather then omitted
                ifndef _ICPRT_OFF_4L
_ICPRT_OFF_4L   EQU     H'DF' 
                endif
              __CONFIG _CONFIG4L, _DEBUG_OFF_4L   & _XINST_OFF_4L        & _ICPRT_OFF_4L   & _LVP_OFF_4L  & _STVREN_ON_4L

              __CONFIG _CONFIG5L, _CP3_OFF_5L     & _CP2_OFF_5L          & _CP1_OFF_5L     & _CP0_OFF_5L
              __CONFIG _CONFIG5H, _CPD_OFF_5H     & _CPB_OFF_5H
              
              __CONFIG _CONFIG6L, _WRT3_OFF_6L    & _WRT2_OFF_6L         & _WRT1_OFF_6L    & _WRT0_OFF_6L
              __CONFIG _CONFIG6H, _WRTD_OFF_6H    & _WRTB_OFF_6H         & _WRTC_OFF_6H
              
              __CONFIG _CONFIG7L, _EBTR3_OFF_7L   & _EBTR2_OFF_7L        & _EBTR1_OFF_7L   & _EBTR0_OFF_7L
              __CONFIG _CONFIG7H, _EBTRB_OFF_7H
;
; Numerical Constants (DEFINES)
;
DELAYTIML       EQU     0x01
DELAYTIMM       EQU     0x01
DELAYTIMH       EQU     0x01
;
; Data Constants (USERID)
;
                ORG     0x200000
                DB      0,1,2,3,4,5,6,7
;
; Data Constants (EEPROM)
;
                ORG     0xF00000
                DE      "Triangle Demo"
;
; Variables
;
;
OUTPUT          EQU     0x20    ;OUTPUT SIGNAL
;
COUNTER1        EQU     0x30    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x31
COUNTER3        EQU     0x32
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            CLRF    ADCON0              ;A/D OFF?
                MOVLW   0x7f
                MOVWF   ADCON1
                CLRF    ADCON2
                MOVLW   b'00000000'
                MOVWF   TRISB
                RETURN
;
; Main program
;
; TRIANGLE WAVE @ 11.7 KHz
;
MAIN            CALL    INIT

MAIN0           CLRF    PORTB               ; 0x00

MAIN1           INCF    PORTB,F             ; 0x01 => 0x80
                BTFSS   PORTB,7
                GOTO    MAIN1
                
                NOP
                NOP

MAIN2           DECF    PORTB,F             ; 0x7f => 0x00
                BTFSS   STATUS,Z
                GOTO    MAIN2

MAIN3           GOTO    MAIN1
;
; Interrupt service routine (unused)
;
INTSR           RETFIE
;
; Code End
;
                END
