;
; Velleman K8048 Test
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: 8_pic16f877a.asm,v 1.6 2007/05/15 20:08:23 darron Exp $
;
; ICSP
; ----
;
; 5 PGC --------> PIN 39
; 4 PGD <-------> PIN 40
; 3 GND --------> PIN 12 & PIN 31
; 2  5V --------> PIN 11 & PIN 32
; 1 12V --------> PIN 1
;
; Additional setup:
;   15pf CERAMIC CAP from PIN 39 to GND
;   15pf CERAMIC CAP from PIN 13 to GND
;   15pf CERAMIC CAP from PIN 14 to GND
;   4MHZ XTAL between PIN 13 & PIN 14
;   LED via ~2K RESISTOR to GND from PIN 33
;   100n CERAMIC CAP between PIN 31 & PIN 32
;
; If all goes well, LED will pulsate
;
                LIST    P=PIC16F877a
#INCLUDE        "p16f877a.inc"
;*****************************************************************************
;
; Configuration word
;
              __CONFIG  _CP_OFF & _DEBUG_OFF & _WRT_OFF  & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _XT_OSC
;             __CONFIG  _CP_ON  & _DEBUG_OFF & _WRT_HALF & _CPD_ON  & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _XT_OSC
;
; Numerical Constants (DEFINES)
;
DELAYTIMH       EQU     0xc0
;
; Data Constants (USERID)
;
                ORG     0x2000
                DE      "PIC!"
;
; Data Constants (EEPROM)
;
                ORG     0x2100
                DE      "EEPROM WRITE TEST",0
;
; Variables
;
DELAYTIMER1     EQU     0x20
DELAYTIMER2     EQU     0x21
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
                ORG     0x0004
                GOTO    INTSR
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
DELAYLOOP       MOVWF   DELAYTIMER1
DELAYLOOP1      CLRF    DELAYTIMER2
DELAYLOOP2      DJNZ    DELAYTIMER2,DELAYLOOP2
                DJNZ    DELAYTIMER1,DELAYLOOP1
                RETURN
;
; Initialise PICMicro
;
INIT            BSF     STATUS,RP0          ;SELECT BANK 1

                MOVLW   B'10000000'         ;SET OPTIONS (DISABLE PULLUPS)
                MOVWF   OPTION_REG

                MOVLW   B'11111111'         ;PORT A ALL I/P
                MOVWF   TRISA
                
                MOVLW   B'11000000'         ;PORT B LD1..LD6 O/P
                MOVWF   TRISB

                CLRF    INTCON              ;DISABLE INTERRUPTS

                BCF     STATUS,RP0          ;SELECT BANK 0

                MOVLW   B'00000111'         ;DISABLE COMPARATORS
                MOVWF   CMCON

                RETURN
;
; Main program
;
MAIN            CALL    INIT                ;INITIALISE

MAIN1           CLRF    PORTB               ;CLEAR PORT

MAIN2           CALL    DELAY               ;DELAY SOME TIME
                INCF    PORTB               ;INCREMENT PORT
                GOTO    MAIN2
;
; Interrupt service routine (unused)
;
INTSR           RETFIE
;
; Code End
;
                END
