;
; Velleman K8048 Test
;
; Copyright (c) 2005-2007 Darron Broad
; All rights reserved.
;
; Licensed under the terms of the BSD license, see file LICENSE
; for details.
;
; $Id: 9_pic18f1320.asm,v 1.5 2007/05/15 20:08:23 darron Exp $
;
                LIST    P=PIC18F1320
#INCLUDE        "p18f1320.inc"
;*****************************************************************************
;
;   k8048 PIC18F1320 Test
;
;   PIC18F1320 port pins differ to the PIC16F device types and subsequently
;   this affects the programming of the k8048 LEDs and switches.
;
;   PIN PIC16F648A  PIC18F1320  K8048
;    1  RA2         RA0         SW3
;    2  RA3         RA1         SW4
;    3  RA4      =  RA4
;    4  RA5         RA5
;    5  Vss         Vss
;    6  RB0         RA2         LD1
;    7  RB1         RA3         LD2
;    8  RB2         RB0         LD3
;    9  RB3         RB1         LD4
;   10  RB4      =  RB4         LD5
;   11  RB5      =  RB5         LD6
;   12  RB6      =  RB6
;   13  RB7      =  RB7
;   14  Vdd         Vdd
;   15  RA6      =  RA6
;   16  RA7      =  RA7
;   17  RA0         RB2         SW1
;   18  RA1         RB3         SW2
;
;   PIC18F1320 TRISA= b'XXXX0011'
;              TRISB= b'XX001100
;
;*****************************************************************************
;
; Configuration words
;
              __CONFIG _CONFIG1H, _IESO_OFF_1H  & _FSCM_OFF_1H & _XT_OSC_1H

              __CONFIG _CONFIG2L, _BORV_20_2L   & _BOR_OFF_2L  & _PWRT_ON_2L
              __CONFIG _CONFIG2H, _WDTPS_32K_2H & _WDT_OFF_2H

              __CONFIG _CONFIG3H, _MCLRE_ON_3H

              __CONFIG _CONFIG4L, _DEBUG_OFF_4L & _LVP_OFF_4L  & _STVR_ON_4L

              __CONFIG _CONFIG5L, _CP1_OFF_5L   & _CP0_OFF_5L
              __CONFIG _CONFIG5H, _CPD_OFF_5H   & _CPB_OFF_5H
              
              __CONFIG _CONFIG6L, _WRT1_OFF_6L  & _WRT0_OFF_6L
              __CONFIG _CONFIG6H, _WRTD_OFF_6H  & _WRTB_OFF_6H & _WRTC_OFF_6H
              
              __CONFIG _CONFIG7L, _EBTR1_OFF_7L & _EBTR0_OFF_7L
              __CONFIG _CONFIG7H, _EBTRB_OFF_7H
;
; Numerical Constants (DEFINES)
;
DELAYTIML       EQU     0x00
DELAYTIMM       EQU     0x00
DELAYTIMH       EQU     0x02
;
; Data Constants (USERID)
;
                ORG     0x200000
                DB      0,1,2,3,4,5,6,7
;
; Data Constants (EEPROM)
;
                ORG     0xF00000
                DE      "Test"
;
; Variables
;
COUNTER0        EQU     0x20
;
COUNTER1        EQU     0x30    ;GENERAL PURPOSE COUNTERS
COUNTER2        EQU     0x31
COUNTER3        EQU     0x32
;
; Macros
;
; decrement REG, goto LOC if not ZERO
DJNZ            MACRO   REG,LOC
                DECFSZ  REG,F
                GOTO    LOC
                ENDM
;
; Code Start
;
                ORG     0x0000
                GOTO    MAIN
;
; Simple delay
;
DELAY           MOVLW   DELAYTIMH
                MOVWF   COUNTER3
DELAYLOOPH      MOVLW   DELAYTIMM
                MOVWF   COUNTER2
DELAYLOOPM      MOVLW   DELAYTIML
                MOVWF   COUNTER1
DELAYLOOPL      DJNZ    COUNTER1,DELAYLOOPL
                DJNZ    COUNTER2,DELAYLOOPM
                DJNZ    COUNTER3,DELAYLOOPH
                RETURN
;
; Initialise PICMicro
;
INIT            CLRF    ADCON0              ;A/D OFF
                MOVLW   0x7f
                MOVWF   ADCON1
                CLRF    ADCON2

                MOVLW   b'11110011'         ;X,X,X,X,LD2,LD1,SW4,SW3
                MOVWF   TRISA
                MOVLW   b'11001100'         ;X,X,LD6,LD5,SW2,SW1,LD4.LD3
                MOVWF   TRISB

                CLRF    PORTA
                CLRF    PORTB
                CLRF    COUNTER0

                RETURN
;
; Main program
;
MAIN            CALL    INIT

MAIN1           INCF    COUNTER0
;
;remap LEDs
                CLRW
                BTFSC   COUNTER0,0
                IORLW   0x04
                BTFSC   COUNTER0,1
                IORLW   0x08
                MOVWF   PORTA               ;WRITE BITS 0/1 LD1/LD2
                CLRW
                BTFSC   COUNTER0,2
                IORLW   0x01
                BTFSC   COUNTER0,3
                IORLW   0x02
                BTFSC   COUNTER0,4
                IORLW   0x10
                BTFSC   COUNTER0,5
                IORLW   0x20
                MOVWF   PORTB               ;WRITE BITS 2/3/4/5 LD3,LD4,LD5,LD6

                CALL    DELAY

                GOTO    MAIN1
;
; Interrupt service routine (unused)
;
INTSR           RETFIE
;
; Code End
;
                END
