/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2007 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: io.h,v 1.27 2008/09/19 04:42:20 darron Exp $
 */

#ifndef _IO_H
#define _IO_H

/*
 * logic
 */
#define LOW (0)
#define HIGH (1)

/*
 * i/o control bit flags
 */
#define FLIPDTR  (1)
#define FLIPRTS  (2)
#define FLIPTX   (4)
#define FLIPCTS  (8)
#define ENABLEPULLUP (16)
#define K8048 (FLIPDTR | FLIPRTS | FLIPTX | FLIPCTS | ENABLEPULLUP)
#define K8076 (ENABLEPULLUP)

/*
 * debug
 */
#define DEBUGTIMER (500)

/*
 * prototypes
 */
int io_open(struct k8048 *);
void io_close(struct k8048 *);

void io_set_pgd(struct k8048 *, int);
void io_set_pgc(struct k8048 *, int);
void io_set_pgd_pgc(struct k8048 *, int, int);
void io_set_mclr(struct k8048 *, int);
unsigned char get_pgd(struct k8048 *);

void io_standby(struct k8048 *);
void io_init_program_verify(struct k8048 *);

void io_data_input_pullup(struct k8048 *);

void io_clock_out(struct k8048 *, int);
unsigned char io_clock_in(struct k8048 *);
void io_command_out(struct k8048 *, char *);

void io_word_out14(struct k8048 *, unsigned short);
unsigned short io_word_in14(struct k8048 *);

void io_word_out(struct k8048 *, unsigned short);
unsigned char io_byte_in(struct k8048 *);

void io_debug(struct k8048 *);
void io_debug_signal();

void io_test1(struct k8048 *, int);
void io_test2(struct k8048 *, int);

void io_cursor(char);

#endif /* !_IO_H */
