/*
 * Velleman K8048 Programmer for FreeBSD and others.
 *
 * Copyright (c) 2005-2007 Darron Broad
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: util.c,v 1.3 2007/05/15 01:06:21 darron Exp $
 */

#include "k8048.h"

/******************************************************************************
 * UTILITY FUNCTIONS
 *****************************************************************************/

/*
 * strcasestr implementation for cygwin etc
 */
const char *
mystrcasestr(const char *haystack, const char *needle)
{
    int i, j;

    for(i=0; i<strlen(haystack); i++)
    {
        for(j=0; j<strlen(needle); j++)
        {
            if( tolower(haystack[i + j]) != tolower(needle[j]) )
                break;
        }
        if(j==strlen(needle))
        {
            return &haystack[i];
        }
    }
    return NULL;
}
    
/*
 * get quoted command line argument
 */
char *
getarg(const char *line)
{
    int len=strlen(line);
    if(len==0)
        return NULL;

    char *arg= calloc(1 + len, sizeof(char));
    if(arg==NULL)
    {
        printf("%s: fatal error: calloc failed\n", __FUNCTION__);
        exit(-1);
    }

    int i=0;
    while(line[i] && line[i]==' ')
        i++;

    char q;
    if(line[i]=='\'' || line[i]=='"')
        q=line[i++];
    else
        q=' ';

    int j=0;
    while(line[i] && line[i]!=q)
        arg[j++]= line[i++];

    if(j==0)
    {
        free(arg);
        return NULL;
    }
    arg[j]='\0';

    return arg;
}
